/*
   Parallel search for "aardvark" in the stdin stream.

   This version does I/O using a Scanner wrapped around either
   System.in or a BufferedInputStream wrapped around System.in.
   (The difference between them is that the buffer in System.in
   is fixed in size, but we can set the size of the bufer in the
   BufferedInputStream.)

   Each line returned by the Scanner is passed to a parallel
   task handler. The task handler splits its line and then
   compares each token to "aardvark".
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.io.IOException;
import java.io.BufferedInputStream;
import java.util.Scanner;

public class SearchParallel_1_stdin
{
   private static final int BUFFERSIZE = 16 * 8192;
   public static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws IOException
   {
     // Compare wrapping a BufferedInputStream around System.in
//      BufferedInputStream bis = new BufferedInputStream(System.in, BUFFERSIZE);
//      Scanner scanner = new Scanner(bis);
      // with using System.in directly
      Scanner scanner = new Scanner(System.in);

      ExecutorService pool = Executors.newFixedThreadPool(12);

      System.out.println(ft.format(new Date()) + " SearchParallel_1_stdin");
      Instant start = Instant.now();
      int lineNumber = 0;

      while ( scanner.hasNextLine() ) // this blocks
      {
         ++lineNumber;
         pool.execute( new Task(scanner.nextLine(), lineNumber) );
      }
//      bis.close();
      System.out.println(ft.format(new Date()) + " I/O completed (processed " + lineNumber + " lines).");
      pool.shutdown();
      try
      {
         pool.awaitTermination(5, TimeUnit.MINUTES);
      }
      catch (InterruptedException e)
      {
         pool.shutdownNow();
         Thread.currentThread().interrupt(); // Preserve interrupt status
      }

      Instant stop = Instant.now();
      System.out.println(ft.format(new Date()));
      long time = Duration.between(start, stop).toMillis();
      System.out.println("elapsed time: " + time + "ms");
      System.out.println( pool.toString() );
   }
}


// A Task object, when it is called by one of the threads in
// the Executor's thread pool, will filter one line of text.
class Task implements Runnable
{
   private final String oneLine;
   private final int lineNumber;

   public Task(final String oneLine, final int lineNumber)
   {
      this.oneLine = oneLine;
      this.lineNumber = lineNumber;
   }

   public void run()
   {
      // Filter the line of input.
      String[] tokens =  oneLine.trim().split("[\\p{Punct}\\s]+");

      for (String s : tokens)
      {
       //if ( s.equals("aardvark") )
         if ( s.indexOf("aardvark") != -1 )
         {
            System.out.println(SearchParallel_1_stdin.ft.format(new Date()) + " " + lineNumber);
         }
      }
   }
}//Task
