/*
   Parallel search for "aardvark" in one file called "data".

   This version does file I/O using a Scanner wrapped around
   a BufferedInputStream wrapped around a FileInputStream.

   Each line returned by the Scanner is passed to a parallel
   task handler. The task handler searches its String sequentially
   for "aardvark".
*/

import java.time.Instant;
import java.time.Duration;
import java.util.Date;
import java.text.SimpleDateFormat;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;
import java.util.concurrent.TimeUnit;
import java.io.IOException;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.BufferedInputStream;
import java.util.Scanner;

public class SearchParallel_2_File
{
   private static final int BUFFERSIZE = 16 * 8192;
   public static final SimpleDateFormat ft = new SimpleDateFormat("hh:mm:ss");

   public static void main(String[] args) throws IOException, FileNotFoundException
   {
      // Open the file.
      BufferedInputStream bis = new BufferedInputStream(
                                new FileInputStream("data"), BUFFERSIZE);
      Scanner scanner = new Scanner(bis);

      ExecutorService pool = Executors.newFixedThreadPool(12);

      System.out.println(ft.format(new Date()) + " SearchParallel_2_File");
      Instant start = Instant.now();
      int lineNumber = 0;

      while ( scanner.hasNextLine() ) // this blocks
      {
         ++lineNumber;
         pool.execute( new Task(scanner.nextLine(), lineNumber) );
      }
      bis.close();
      System.out.println(ft.format(new Date()) + " I/O completed (processed " + lineNumber + " lines).");
      pool.shutdown();
      try
      {
         pool.awaitTermination(5, TimeUnit.MINUTES);
      }
      catch (InterruptedException e)
      {
         pool.shutdownNow();
         Thread.currentThread().interrupt(); // Preserve interrupt status
      }

      Instant stop = Instant.now();
      System.out.println(ft.format(new Date()));
      long time = Duration.between(start, stop).toMillis();
      System.out.println("elapsed time: " + time + "ms");
      System.out.println( pool.toString() );
   }
}


// A Task object, when it is called by one of the threads in
// the Executor's thread pool, will filter one line of text.
class Task implements Runnable
{
   private final String oneLine;
   private final int lineNumber;

   public Task(final String oneLine, final int lineNumber)
   {
      this.oneLine = oneLine;
      this.lineNumber = lineNumber;
   }

   public void run()
   {
      // Filter the line of input.
      for (int i = 0; i < oneLine.length() - 8; i++)
      {
         if ( 'a' == oneLine.charAt(i)
           && 'a' == oneLine.charAt(i+1)
           && 'r' == oneLine.charAt(i+2)
           && 'd' == oneLine.charAt(i+3)
           && 'v' == oneLine.charAt(i+4)
           && 'a' == oneLine.charAt(i+5)
           && 'r' == oneLine.charAt(i+6)
           && 'k' == oneLine.charAt(i+7) )
         {
            System.out.println(SearchParallel_2_File.ft.format(new Date()) + " " + lineNumber);
         }
      }
   }
}//Task
